searchData={"items":[{"type":"module","title":"Spannex","doc":"The entry module for interacting with `Spannex.Protocol`, the underlying `DBConnection` implementation.\n\nTo create a Spannex database connection to Cloud Spanner, simply call `Spannex.start_link/1` with the database configuration or pass in `{Spannex, opts}` to your supervisor.","ref":"Spannex.html"},{"type":"module","title":"Queries vs Mutations - Spannex","doc":"Spanner makes a clear distinction between \"queries\", which are read-only statements, and \"mutations\", which are statements that modify the state of the database. However, Spannex still provides a single function, `execute/4`, and will perform any necessary internal conversions depending on the type of the statement being executed.","ref":"Spannex.html#module-queries-vs-mutations"},{"type":"module","title":"Named Variables - Spannex","doc":"Currently, only named variables are accepted in queries and mutations. You must also pass in a map of named variables when executing a statement that contains named variables. For example, if you have a query like `SELECT * FROM users WHERE id = @id`, you must pass in a map containing `%{id: 1}` when executing the statement.","ref":"Spannex.html#module-named-variables"},{"type":"function","title":"Spannex.child_spec/1","doc":"Returns a child specification for the Spannex database connection using the given options.\n\nYou *must* supply at least one option, `:database`, to specify the Cloud Spanner database to connect to.\n\nIt is recommended to avoid supplying the `:name` option and to instead let the connection pool handle fetching and returning individual connections.\n\nSee `Spannex.Protocol.connect/1` for a full list of accepted options.","ref":"Spannex.html#child_spec/1"},{"type":"function","title":"Spannex.execute/4","doc":"Converts a string SQL statement into a `Spannex.Query` struct before calling `DBConnection.prepare_execute/4`.","ref":"Spannex.html#execute/4"},{"type":"function","title":"Spannex.start_link/1","doc":"Starts a Spannex database connection using the given options.\n\nSynonymous with `DBConnection.start_link(Spannex.Protocol, opts)`.\n\nYou *must* supply at least one option, `:database`, to specify the Cloud Spanner database to connect to.\n\nSee `Spannex.Protocol.connect/1` for a full list of accepted options.","ref":"Spannex.html#start_link/1"},{"type":"module","title":"Spannex.Protocol","doc":"A `DBConnection` implementation for Cloud Spanner.","ref":"Spannex.Protocol.html"},{"type":"module","title":"Connections - Spannex.Protocol","doc":"Each connection maintains its own gRPC channel to the Spanner API and its own unique session. The session is used as a unique identifier for all other requests made against the database.","ref":"Spannex.Protocol.html#module-connections"},{"type":"module","title":"Transactions - Spannex.Protocol","doc":"When starting a transaction, there are four accepted types of transactions:\n\n1. `:batch_write` - Uses \"batch write\" and \"mutation groups\" to execute multiple operations in a single transaction, optimized for \"blind bulk writes\", low-latency, and atomicity within each group of mutations.\n2. `:read_write` - A locking read-write transaction, this is what you would consider a \"traditional transaction\". It's best used when you are changing one row or a handful of related rows or need to reference data created in the same transaction.\n3. `:read_only` - A special transaction type that allows for point-in-time consistency in the data being read. This helps eliminate possible drift in values when operations might be happening in parallel to the read-only transaction by locking the state of the data being read to a specific point in time.\n4. `:partitioned_dml` - A special transaction where regular DML statements are partitioned and executed in parallel. This is useful for large-scale bulk operations that still need guarantees of atomicity and consistency.\n\nThe `:batch_write` transaction differs from `:read_write` and `:partitioned_dml` transactions in some important ways:\n\n1. It does not support reading data. It is strictly for writing data.\n2. You cannot reference changes made by other mutations within the same transaction.\n3. You must specify all literals explicitly, you cannot use named query parameters.\n4. All queries are locally queued and then sent all at once when the transaction is committed. This means individual statements are not sent to the database, rather they are batched into a single gRPC request when the transaction is comitted.\n5. Deletions *must* be done by primary key. If you need a custom where-clause to delete rows, you must do so in a `:read_write` or `:partitioned_dml` transaction using regular DML statements.\n\n#","ref":"Spannex.Protocol.html#module-transactions"},{"type":"module","title":"Timeouts - Spannex.Protocol","doc":"Transactions have a 10-second idle timeout that cannot be modified. If a transaction remains idle for more than 10 seconds, it is aborted. Aborted transactions cause the session to enter a limbo state where you can't really do anything useful anymore, so the default behavior is to disconnect and force the creation of a new connection.\n\nWith the exception of a `:batch_write` transactions, all individual statements resolved via `handle_execute` will reset the 10-second idle timer. But if you start a `:batch_write` transaction, you have 10 seconds to queue up all necessary statements and then commit the transaction. While this should be sufficient in most scenarios, it is worth keeping this in mind if you are using `:batch_write` transactions for very large operations. In such cases where the 10-second idle timeout might be a problem, consider pre-calculating as much data for the DML statements as possible and then starting a `:batch_write` transaction after.","ref":"Spannex.Protocol.html#module-timeouts"},{"type":"module","title":"Queries - Spannex.Protocol","doc":"Spanner requires that all write operations, i.e. queries that are not `SELECT` statements, use an explicit transaction. If you attempt to execute a write statement outside of a transaction, you will receive an error.\n\nFor read queries, i.e. `SELECT` statements, a temporary read-only transaction with a strong consistency guarantee can be created for you.\n\nSpanner also stores version history for all columns and all rows and all tables, meaning you can get truly strong consistency guarantees when reading data. You can ensure that, given a specific (recent) point in time, all data read from the database is no more recent than that point in time.","ref":"Spannex.Protocol.html#module-queries"},{"type":"function","title":"Spannex.Protocol.checkout/1","doc":"No-op","ref":"Spannex.Protocol.html#checkout/1"},{"type":"function","title":"Spannex.Protocol.connect/1","doc":"Start a new gRPC connection to the Spanner API and also generate a new session.\n\nThere is only one required option for the `connect/1` function: `:database`. This tells Spannex which database the session will be for.\n\nYou may optionally supply the following other options:\n\n- `:labels` - A map of labels to apply to the session. These labels can be used to filter and monitor sessions using the Cloud console or admin API.\n- `:host` - The gRPC endpoint to connect to. If not supplied, the global Spanner API endpoint is used.\n- `:grpc_opts` - The options to pass to `GRPC.Stub.connect/2`.\n- `:cred` - A gRPC credential wrapper. If not supplied, the `Goth` library is used to fetch a Google Cloud access token.\n- `:headers` - A list of headers to use for the connection. If not supplied, `content-type` and `authorization` headers will be set to sane defaults.","ref":"Spannex.Protocol.html#connect/1"},{"type":"function","title":"Spannex.Protocol.convert_value/3","doc":"Converts a single column value from its gRPC-encoded form to its Elixir-native representation. The `type` parameter is the native type of the column and the `enc_type` parameter is the type of the encoded value.","ref":"Spannex.Protocol.html#convert_value/3"},{"type":"function","title":"Spannex.Protocol.handle_begin/2","doc":"Begins a new transaction. You *must* use a transaction for any mutations, i.e. anything that isn't a `SELECT` query.\n\nMay specify the option `:type` as any one of: `:batch_write`, `:read_write`, `:read_only`, `:partitioned_dml`. If `:type` is not set, it defaults to `:read_write`.\n\nIf `:batch_write` is specified, all subsequent `handle_execute/4` calls will simply queue up the statements locally in a list that will be converted to a set of mutations when `handle_commit/2` is called. If `handle_rollback/2` is called instead, the queue is simply cleared, nothing happens remotely.","ref":"Spannex.Protocol.html#handle_begin/2"},{"type":"function","title":"Spannex.Protocol.handle_close/3","doc":"No-op","ref":"Spannex.Protocol.html#handle_close/3"},{"type":"function","title":"Spannex.Protocol.handle_commit/2","doc":"Commits a running transaction.\n\nIf the transaction is a `:batch_write` transaction, all queued statements are converted to \"mutations\" and sent to the Spanner API in a single request.","ref":"Spannex.Protocol.html#handle_commit/2"},{"type":"function","title":"Spannex.Protocol.handle_deallocate/4","doc":"Would normally deallocate a cursor generated by `handle_declare/4`, but we don't support cursors in this implementation.","ref":"Spannex.Protocol.html#handle_deallocate/4"},{"type":"function","title":"Spannex.Protocol.handle_declare/4","doc":"Would normally generate a cursor for a query prepared by `handle_prepare/3`, but we don't support cursors in this implementation.","ref":"Spannex.Protocol.html#handle_declare/4"},{"type":"function","title":"Spannex.Protocol.handle_execute/4","doc":"Executes a SQL query against the Spanner API.\n\nIf the query is a `SELECT` statement, it will be executed in a temporary read-only transaction.\n\nAll other queries must be executed within a transaction.\n\nIf there is a running `:batch_write` transaction, the query will be queued up locally and not sent to the Spanner API until `handle_commit/2` is called.","ref":"Spannex.Protocol.html#handle_execute/4"},{"type":"function","title":"Spannex.Protocol.handle_fetch/4","doc":"Would normally fetch the next result from a cursor generated by `handle_declare/4`, but we don't support cursors in this implementation.","ref":"Spannex.Protocol.html#handle_fetch/4"},{"type":"function","title":"Spannex.Protocol.handle_prepare/3","doc":"No-op","ref":"Spannex.Protocol.html#handle_prepare/3"},{"type":"function","title":"Spannex.Protocol.handle_rollback/2","doc":"Rolls back a running transaction.\n\nIf the transaction is a `:batch_write` transaction, all queued statements are simply cleared and the state is returned to idle.","ref":"Spannex.Protocol.html#handle_rollback/2"},{"type":"function","title":"Spannex.Protocol.make_mutation/4","doc":"Creates a mutation struct for the given operation. For anything other than a `:delete` operation, the `columns` and `rows` parameters are used to create a `Google.Spanner.V1.Mutation.Write` struct.\n\nFor a `:delete` operation, the Spanner API expects that the primary key will be used to execute the delete mutation. If you need to delete using a where-clause and not the primary key, you cannot use a mutation. The `rows` parameter is expected to be a list of values that correspond to the primary key columns in the same order as they are defined in the table schema.","ref":"Spannex.Protocol.html#make_mutation/4"},{"type":"function","title":"Spannex.Protocol.map_results/1","doc":"Converts the data from a `Google.Spanner.V1.ResultSet` into a list of maps, where each map is one row of the result set. The keys of the map are the column names and the values are the column values. The column values are converted from their encoded form to their Elixir-native representation.","ref":"Spannex.Protocol.html#map_results/1"},{"type":"function","title":"Spannex.Protocol.map_row/3","doc":"Maps a single row of a result set into a map of column names to column values. The column names are taken from the \"fields\" metadata passed as the first argument. The list of column values given as the second argument are converted from their gRPC-encoded form to their Elixir-native representation.\n\n**Important**: An exception will be raised if the number of column metadata items does not match the number of column values.","ref":"Spannex.Protocol.html#map_row/3"},{"type":"function","title":"Spannex.Protocol.ping/1","doc":"No-op","ref":"Spannex.Protocol.html#ping/1"},{"type":"function","title":"Spannex.Protocol.query_to_mutation/1","doc":"Parses a SQL query with optional named parameters and converts the operation to a mutation using `make_mutation/4`.","ref":"Spannex.Protocol.html#query_to_mutation/1"},{"type":"type","title":"Spannex.Protocol.t/0","doc":"","ref":"Spannex.Protocol.html#t:t/0"},{"type":"type","title":"Spannex.Protocol.transaction_type/0","doc":"","ref":"Spannex.Protocol.html#t:transaction_type/0"},{"type":"module","title":"Spannex.Query","doc":"Handles parsing and preparing queries for use by the Spanner database connection protocol.","ref":"Spannex.Query.html"},{"type":"type","title":"Spannex.Query.t/0","doc":"","ref":"Spannex.Query.html#t:t/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}